# PHYS2041 - Atomic Spectroscopy
# Aqeel Akber
# Requires Octave package: Physical Constants
# 
#
#Collect Given Sodium data
# We are given some Sodium data to help completel our emperical model.
# Here we are collecting all that data and organising in such a way that
# it is easier to call later onwards.
# No uncertainty was given for this data; so in order to get a reasonable
# estimate NIST data from 2008-2009 was used as a comparison.
#
#Given Sodium data
sodium_wl_given = [1139, 819, 330.24, 285.28, 268.04];
sodium_freq_given = physical_constant("SPEED_OF_LIGHT_IN_VACUUM")./(sodium_wl_given*1e-9);
sodium_nrg_given = physical_constant("PLANCK_CONSTANT_IN_EV_S").*sodium_freq_given;
nrg_3s = -5.139; # Given ground state energy of 3s orbital
#
#Estimate error in given data compared to NIST data
sodium_wl_nist = [1138.145, 819.4807,330.237,285.281,268.039];
# Elements 2 and 5 of the above vector were actually doublets; the average was taken.
nrg_3s_nist = -5.139076; # NIST data
sodium_wl_given_error = sodium_wl_given .- sodium_wl_nist;
nrg_3s_error = nrg_3s .- nrg_3s_nist;
#
#Propogate error of given wavelength through to given energy
sodium_freq_given_error = sodium_freq_given.*(sodium_wl_given_error./sodium_wl_given);
sodium_nrg_given_error = sodium_nrg_given.*(sodium_freq_given_error./sodium_freq_given);
#
# This is the end of work on our given data.
#
#Measured data
# From here onwards we are working on our measured data to determine
# any emperical results from analysis of our measurements.
#
#Load measured data
# We must load our emperical data before we can work on it.
load("calibration_data","cali_orig"); #Calibration Measurements
load("sodium_data","sodium_orig"); #Sodium Measurements
#
#Create emperical relationship from calibration measurements
# Firstly we must create a relationship between wavelength and diffraction
# angle based on well known emission lines of mercury. This is an emperical
# relationship used for calibration of our device.
# Both a second degree polynomial and third degree polynomial were fit and 
# compared.
[cali_fit2,cali_fit2_error] = polyfit(cali_orig(:,2),cali_orig(:,1),2);
ste2 = sqrt(diag(inv(cali_fit2_error.R)*inv(cali_fit2_error.R'))./cali_fit2_error.normr.^2./cali_fit2_error.df);
[cali_fit3,cali_fit3_error] = polyfit(cali_orig(:,2),cali_orig(:,1),3);
ste3 = sqrt(diag(inv(cali_fit3_error.R)*inv(cali_fit3_error.R'))./cali_fit3_error.normr.^2./cali_fit3_error.df);
cali_fit_x=[174:0.1:185];
cali_fit2_plot = polyval(cali_fit2,cali_fit_x);
cali_fit3_plot = polyval(cali_fit3,cali_fit_x); 
# Degree 3 fit is better; we're using this from now on.
#
#Calculate Sodium Wavelengths with emperical relationship
# Now that we know how our device behaves we can determine unknown wavelenths
# for a given angle. Here we calculate the sodium wavelengths after taking the mean
# of the all the trials. The standard error is also calculated.
sodium_mean = mean(transpose(sodium_orig));
sodium_mean_ste = std(transpose(sodium_orig))/sqrt(length(transpose(sodium_orig)));
[sodium_wl3, sodium_wl3_error] = polyval(cali_fit3,sodium_mean,cali_fit3_error);
#
#Calculate Sodium Frequency
# Frequency is related to wavelength by the speed of the EM wave.
# As the distances are small the difference between the speed of light
# in a vacuum and that in lab conditions on Earth are negligible when 
# time is calculated. 
# Physical constant is in metres per second therefore wavelength must
# be converted to metres if it already is not so.
sodium_freq3 = physical_constant("SPEED_OF_LIGHT_IN_VACUUM")./(sodium_wl3*1e-9);
sodium_freq3_error = sodium_freq3.*(sodium_wl3_error./sodium_wl3);
#
#Calculate Sodium Photon Energies in eV
# Energy is related to the frequency by the Quantum relationship with 
# Planck's constant. As we expect very small energies we are using Planck's
# constant in eV.
sodium_nrg3 = physical_constant("PLANCK_CONSTANT_IN_EV_S").*sodium_freq3;
sodium_nrg3_error = sodium_nrg3.*(sodium_freq3_error./sodium_freq3);
#
#Calculate Energy Levels in eV from measured data - Conservation of Energy
# As we know the ground state energy of the sodium atom and we are given
# the known transitions of electrons which emit discrete photons we can
# calculate energies of the excited states. This is a conservation of energy.
nrg_3p = nrg_3s + sodium_nrg3(1,2); # Calculated from Yellow
nrg_5s = nrg_3p + sodium_nrg3(1,1); # Calculated from Red
nrg_4d = nrg_3p + sodium_nrg3(1,3); # Calculated from Green1
nrg_6s = nrg_3p + sodium_nrg3(1,4); # Calculated from Green2
nrg_5d = nrg_3p + sodium_nrg3(1,5); # Calculated from Green3
nrg_7s = nrg_3p + sodium_nrg3(1,6); # Calculated from Blue-Green
nrg_6d = nrg_3p + sodium_nrg3(1,7); # Calculated from Blue
nrg_8s = nrg_3p + sodium_nrg3(1,8); # Calculated from Violet - Very weak
#
#Calculate Energy Levels in eV from given data - Conservation of Energy
# As per above.
nrg_4s = nrg_3p + sodium_nrg_given(1,1); # Given from 1139 nm
nrg_3d = nrg_3p + sodium_nrg_given(1,2); # Given from 819 nm
nrg_4p = nrg_3s + sodium_nrg_given(1,3); # Given from 330.24 nm
nrg_5p = nrg_3s + sodium_nrg_given(1,4); # Given from 285.28 nm
nrg_6p = nrg_3s + sodium_nrg_given(1,5); # Given from 268.04 nm
#
#Calculate error in energy
# Now we must propagate through the error in these calculations.
#Error in Measured data energies
nrg_3p_error = sqrt(nrg_3s_error^2 + sodium_nrg3_error(1,2)^2); 
nrg_5s_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,1)^2);
nrg_4d_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,3)^2);
nrg_6s_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,4)^2);
nrg_5d_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,5)^2);
nrg_7s_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,6)^2);
nrg_6d_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,7)^2);
nrg_8s_error = sqrt(nrg_3p_error^2 + sodium_nrg3_error(1,8)^2);
#
#Error in Given data energies
nrg_4s_error = sqrt(nrg_3p_error^2 + sodium_nrg_given_error(1,1)^2);
nrg_3d_error = sqrt(nrg_3p_error^2 + sodium_nrg_given_error(1,2)^2);
nrg_4p_error = sqrt(nrg_3s_error^2 + sodium_nrg_given_error(1,3)^2);
nrg_5p_error = sqrt(nrg_3s_error^2 + sodium_nrg_given_error(1,4)^2);
nrg_6p_error = sqrt(nrg_3s_error^2 + sodium_nrg_given_error(1,5)^2);
#
#Organise the energies
# To make life easier when plotting and analysing here we are creating
# new vectors which sort the energy data the way we want it.
nrg_s = [nrg_3s, nrg_4s, nrg_5s, nrg_6s, nrg_7s, nrg_8s];
nrg_p = [nrg_3p, nrg_4p, nrg_5p, nrg_6p];
nrg_d = [nrg_3d, nrg_4d, nrg_5d, nrg_6d];
nrg_s_error = [nrg_3s_error, nrg_4s_error, nrg_5s_error, nrg_6s_error, nrg_7s_error, nrg_8s_error];
nrg_p_error = [nrg_3p_error, nrg_4p_error, nrg_5p_error, nrg_6p_error];
nrg_d_error = [nrg_3d_error, nrg_4d_error, nrg_5d_error, nrg_6d_error];
nrg_s_n = [3, 4, 5, 6, 7, 8];
nrg_p_n = [3, 4, 5, 6];
nrg_d_n = [3, 4, 5, 6];
#
#Linearize energies
# We can linearize the relationshi between energies and principle 
# quantum number, n. By taking the inverse of the square root of the energy.
[lin_nrg_s] = abs((nrg_s)).^-0.5;
[lin_nrg_p] = abs((nrg_p)).^-0.5;
[lin_nrg_d] = abs((nrg_d)).^-0.5;
[lin_nrg_s_error] = nrg_s_error./(2.*abs((nrg_s)).^1.5);
[lin_nrg_p_error] = nrg_p_error./(2.*abs((nrg_p)).^1.5);
[lin_nrg_d_error] = nrg_d_error./(2.*abs((nrg_d)).^1.5);
#
#LSQ weighted linear fit on energy data
# Now that was a linearised relationshi and it is deemed reasonable
# we can do a weighted regression on the data. In this part we're using
# a linear regression funtion made freely available by Edward T Peltzer using
# an equation from Bevington & Robinson (1992) Data Reduction and Error
# Analysis for the Physical Sciences, 2nd Ed and adapted from York (1966) 
# Canad. J. Phys. 44: 1079-1086.
[nrg_s_slope, nrg_s_intercept, nrg_s_rsquare, nrg_s_slope_error, nrg_s_intercept_error] = lsqfityz(nrg_s_n,lin_nrg_s,lin_nrg_s_error);
[nrg_p_slope, nrg_p_intercept, nrg_p_rsquare, nrg_p_slope_error, nrg_p_intercept_error] = lsqfityz(nrg_p_n,lin_nrg_p,lin_nrg_p_error);
[nrg_d_slope, nrg_d_intercept, nrg_d_rsquare, nrg_d_slope_error, nrg_d_intercept_error] = lsqfityz(nrg_d_n,lin_nrg_d,lin_nrg_d_error);
nrg_s_fit = [nrg_s_slope, nrg_s_intercept];
nrg_p_fit = [nrg_p_slope, nrg_p_intercept];
nrg_d_fit = [nrg_d_slope, nrg_d_intercept];
nrg_plot_x = [2:9];
nrg_s_plot = polyval(nrg_s_fit,nrg_plot_x);
nrg_p_plot = polyval(nrg_p_fit,nrg_plot_x);
nrg_d_plot = polyval(nrg_d_fit,nrg_plot_x);
#
#
#Plotting and saving figures
# Here onwards we are plotting and saving figures for importing in LaTeX.
#Plot emperical relationship and calibration data on the same plot and save
clf
set (gca, "xminortick", "on", "yminortick", "on")
plot(cali_orig(:,2),cali_orig(:,1),"markersize",10,"+",
cali_fit_x,cali_fit2_plot,"-",cali_fit_x,cali_fit3_plot,"-");
hold on
title("\\large \\bf Calibration of 7.5 inch prism based manual spectroscope using Hg lamp");
xlabel("Measured angle of diffraction: $\\theta$ (degrees)");
ylabel("Known emmision line wavelength: $\\lambda$  (nanometres)");
legend("\\footnotesize Known wavelengths",
"\\footnotesize Quadratic LSQ fit","\\footnotesize Cubic LSQ fit");
print('-dfig', '-color', '-textspecial','calibrationplot1.fig');
#
#Plot linearised energy versus principle quantum number
clf
set (gca, "xminortick", "off", "yminortick", "on")
errorbar(nrg_s_n,lin_nrg_s,lin_nrg_s_error,"~",
nrg_p_n,lin_nrg_p,lin_nrg_p_error,"~",
nrg_d_n,lin_nrg_d,lin_nrg_d_error,"~");
hold on
plot(nrg_plot_x,nrg_s_plot,"-",nrg_plot_x,nrg_p_plot,"-",nrg_plot_x,nrg_d_plot,"-");
axis([1,10,0,2.5]);
title("\\large \\bf Linearised relationship between principle quantum number and electron shell energy");
xlabel("Principle quantum number: $n$ (integer)");
ylabel("Inverse square root energy of electron shell of Sodium atom: $\\frac{1}{\\sqrt {E_\\ell}}$  (eV)");
legend("\\footnotesize $\\ell=s$","\\footnotesize $\\ell=p$",
"\\footnotesize $\\ell=d$","\\footnotesize Weighted LSQ fit: $\\ell=s$",
"\\footnotesize Weighted LSQ fit: $\\ell=p$",
"\\footnotesize Weighted LSQ fit: $\\ell=d$","location","northwest");
print('-dfig', '-color', '-textspecial','energyregression.fig');
#
#Making Grotrian Diagram
#
#
#
#Grotrian energy shell lines
grot_plot_x = [0:0.1:2];
# For l = s
[grot_3s] = [0.2, 0.8 ; nrg_3s, nrg_3s];
[grot_4s] = [0.2, 0.8 ; nrg_4s, nrg_4s];
[grot_5s] = [0.2, 0.8 ; nrg_5s, nrg_5s];
[grot_6s] = [0.2, 0.8 ; nrg_6s, nrg_6s];
[grot_7s] = [0.2, 0.8 ; nrg_7s, nrg_7s];
[grot_8s] = [0.2, 0.8 ; nrg_8s, nrg_8s];
# For l = p
[grot_3p] = [1.2, 1.8 ; nrg_3p, nrg_3p];
[grot_4p] = [1.2, 1.8 ; nrg_4p, nrg_4p];
[grot_5p] = [1.2, 1.8 ; nrg_5p, nrg_5p];
[grot_6p] = [1.2, 1.8 ; nrg_6p, nrg_6p];
# For l = d
[grot_3d] = [2.2, 2.8 ; nrg_3d, nrg_3d];
[grot_4d] = [2.2, 2.8 ; nrg_4d, nrg_4d];
[grot_5d] = [2.2, 2.8 ; nrg_5d, nrg_5d];
[grot_6d] = [2.2, 2.8 ; nrg_6d, nrg_6d];
#Energy transition lines
# To 3s:
[grot_3p_3s] = [1.5, 0.5 ; nrg_3p, nrg_3s];
[grot_4p_3s] = [1.5, 0.5 ; nrg_4p, nrg_3s];
[grot_5p_3s] = [1.5, 0.5 ; nrg_5p, nrg_3s];
[grot_6p_3s] = [1.5, 0.5 ; nrg_6p, nrg_3s];
# To 3p:
[grot_4s_3p] = [0.5, 1.5 ; nrg_4s, nrg_3p];
[grot_5s_3p] = [0.5, 1.5 ; nrg_5s, nrg_3p];
[grot_6s_3p] = [0.5, 1.5 ; nrg_6s, nrg_3p];
[grot_7s_3p] = [0.5, 1.5 ; nrg_7s, nrg_3p];
[grot_8s_3p] = [0.5, 1.5 ; nrg_8s, nrg_3p];
[grot_3d_3p] = [2.5, 1.5 ; nrg_3d, nrg_3p];
[grot_4d_3p] = [2.5, 1.5 ; nrg_4d, nrg_3p];
[grot_5d_3p] = [2.5, 1.5 ; nrg_5d, nrg_3p];
[grot_6d_3p] = [2.5, 1.5 ; nrg_6d, nrg_3p];
#Plotting Grotrian Diagram
clf
set (gca, "xminortick", "off", "yminortick", "on","xtick",[0.5:2.5],
"xticklabel",["s";"p";"d"]);
hold on
# Plotting the energy lines
grot_diag=plot(grot_3s(1,:),grot_3s(2,:),grot_4s(1,:),grot_4s(2,:),grot_5s(1,:),grot_5s(2,:),
grot_6s(1,:),grot_6s(2,:),grot_7s(1,:),grot_7s(2,:),grot_8s(1,:),grot_8s(2,:),
grot_3p(1,:),grot_3p(2,:),grot_4p(1,:),grot_4p(2,:),grot_5p(1,:),grot_5p(2,:),
grot_6p(1,:),grot_6p(2,:),grot_3d(1,:),grot_3d(2,:),grot_4d(1,:),grot_4d(2,:),
grot_5d(1,:),grot_5d(2,:),grot_6d(1,:),grot_6d(2,:));
# Plotting the transition indicators
grot_arrows=plot(grot_3p_3s(1,:),grot_3p_3s(2,:),grot_4p_3s(1,:),grot_4p_3s(2,:),
grot_5p_3s(1,:),grot_5p_3s(2,:),grot_6p_3s(1,:),grot_6p_3s(2,:),
grot_4s_3p(1,:),grot_4s_3p(2,:),grot_5s_3p(1,:),grot_5s_3p(2,:),
grot_6s_3p(1,:),grot_6s_3p(2,:),grot_7s_3p(1,:),grot_7s_3p(2,:),
grot_8s_3p(1,:),grot_8s_3p(2,:),grot_3d_3p(1,:),grot_3d_3p(2,:),
grot_4d_3p(1,:),grot_4d_3p(2,:),grot_5d_3p(1,:),grot_5d_3p(2,:),
grot_6d_3p(1,:),grot_6d_3p(2,:));
# Beautification of diagram
set(grot_diag,"linewidth",4);
set(grot_arrows(1),"linewidth",3);
title("\\large \\bf Grotrian Diagram of empirical sodium atom energy transitions");
xlabel("Azimuthal quantum number: $\\ell$ (integer)");
ylabel("Electron shell energy level: $E$  (eV)");
legend("\\footnotesize $n=3\\; \\ell=s$","\\footnotesize $n=4\\; \\ell=s$",
"\\footnotesize $n=5\\; \\ell=s$","\\footnotesize $n=6\\; \\ell=s$",
"\\footnotesize $n=7\\; \\ell=s$","\\footnotesize $n=8\\; \\ell=s$",
"\\footnotesize $n=3\\; \\ell=p$","\\footnotesize $n=4\\; \\ell=p$",
"\\footnotesize $n=5\\; \\ell=p$","\\footnotesize $n=6\\; \\ell=p$",
"\\footnotesize $n=3\\; \\ell=d$","\\footnotesize $n=4\\; \\ell=d$",
"\\footnotesize $n=5\\; \\ell=d$","\\footnotesize $n=6\\; \\ell=d$","location","east");
axis([0,3]);
print('-dfig', '-color', '-textspecial','grotriandiagram36.fig');
